#!/usr/bin/env python
#coding=utf-8

import onelab, sys

workdir = onelab.path(__file__) 

OL = onelab.client()
    
# Display the geometry in the GUI
if not OL.loop :
    OL.openProject(onelab.path(__file__, 'beam.geo'))
    
MATERIAL = OL.defineNumber('3Material/0Name', value=1,
           labels = {1:'Steel', 2:'Wood', 3:'Aluminium', 4:'User defined'})
YOUNG = OL.defineNumber('3Material/YOUNG', value=100, label='Young modulus [GPa]')
POISSON = OL.defineNumber('3Material/POISSON', value=0.3, label='Poisson ratio []')
DENSITY = OL.defineNumber('3Material/DENSITY', value=6600, label='Mass density [kg/m3]')

if MATERIAL == 1:
    OL.setNumber('3Material/YOUNG', readOnly=1, value=210)
    OL.setNumber('3Material/POISSON', readOnly=1, value=0.3)
    OL.setNumber('3Material/DENSITY', readOnly=1, value=7500)
elif MATERIAL == 2:
    OL.setNumber('3Material/YOUNG', readOnly=1, value=13)
    OL.setNumber('3Material/POISSON', readOnly=1, value=0.45)
    OL.setNumber('3Material/DENSITY', readOnly=1, value=900)
elif MATERIAL == 3:
    OL.setNumber('3Material/YOUNG', readOnly=1, value=70)
    OL.setNumber('3Material/POISSON', readOnly=1, value=0.33)
    OL.setNumber('3Material/DENSITY', readOnly=1, value=6000)
else:
    OL.setNumber('3Material/YOUNG', readOnly=0)
    OL.setNumber('3Material/POISSON', readOnly=0)
    OL.setNumber('3Material/DENSITY', readOnly=0)

WEIGHT = OL.defineNumber('3Material/0WEIGHT', value=1, choices = [0, 1],
                          label='Account for weight')

LOOP = OL.defineNumber('LOOP', value=0, choices = [0, 1],
                          label='Compute MT diagram')
L = OL.defineNumber('1Geometry/L', label='Length - L [m]', value=1)
X = OL.defineNumber('1Geometry/X', label='Cut location [m]', value=0.5,
                     min=L*0.05, max=L*0.95, step=0.05)
    
LOADTYPE = OL.defineNumber('2Load/2LOADTYPE', value=0, label='Load type',
           labels = {0:'None',
                     1:'Uniform',
                     2:'Ramp',
                     3:'Rectangle',
                     4:'Torsion',
                     5:'User defined'})
LOAD = OL.defineNumber('2Load/2LOAD', value=1, label='Magnitude - P [N/cm2]')
D = OL.defineNumber('2Load/D', value=0.1, label='delta d [m]')
XX = OL.defineNumber('2Load/XX', value=0.5, label='Position X [m]')

FORMULA = OL.defineString('2Load/3FORMULA', label="Formula")

if LOADTYPE == 0 :
    OL.setString('2Load/3FORMULA', value='0')
elif LOADTYPE == 1 :
    OL.setString('2Load/3FORMULA', value='P')
elif LOADTYPE == 2 :
    OL.setString('2Load/3FORMULA', value='P *tx(0)/L')
elif LOADTYPE == 3 :
    OL.setString('2Load/3FORMULA', value='if(tx(0)<(X-d)) {0} else {if(tx(0)<=(X+d)) {P} else {0}}')
elif LOADTYPE == 4 :
    OL.setString('2Load/3FORMULA', value='P*tx(1)/B*10')

    
if LOADTYPE == 0 :
    OL.setString('2Load/3FORMULA', visible=0)
    OL.setNumber('2Load/2LOAD', visible=0)
else :
    OL.setString('2Load/3FORMULA', visible=1)
    OL.setNumber('2Load/2LOAD', visible=1)

if LOADTYPE == 3 :
    OL.setNumber('2Load/D', visible=1)
    OL.setNumber('2Load/XX', visible=1)
else :
    OL.setNumber('2Load/D', visible=0)
    OL.setNumber('2Load/XX', visible=0)
     
if LOADTYPE == 5 :
    OL.setString('2Load/3FORMULA', readOnly=0)
else :
    OL.setString('2Load/3FORMULA', readOnly=1)

recomputeElmer = OL.isChanged('mesher') or OL.isChanged(onelab.path(__file__, 'beam.sif.ol'))
OL.sendInfo('Recomputation needed=%g' %(recomputeElmer))


# Check for ONELAB parameter in client files         
OL.run('mesher', 'gmsh - ' +  onelab.path(__file__, 'beam.geo'),'')
OL.preProcess(onelab.path(__file__, 'beam.sif.ol'))

## Define output quantities
VMIN = OL.defineNumber('9Results/VMIN', label='min v(x) [mm]', readOnly=1, 
                       attributes={'Highlight':'Coral'})
VMAX = OL.defineNumber('9Results/VMAX', label='max v(x) [mm]', readOnly=1, 
                       attributes={'Highlight':'Coral'})
M = OL.defineNumber('9Results/2M', label='Moment - M [Nm]', readOnly=1, 
                    attributes={'Highlight':'Coral'})
N = OL.defineNumber('9Results/3N', label='Traction - N [N]', readOnly=1, 
                    attributes={'Highlight':'Coral'})
T = OL.defineNumber('9Results/4T', label='Shear - T [N]', readOnly=1, 
                    attributes={'Highlight':'Coral'})

if LOOP :
    OL.setNumber('1Geometry/X', attributes={'Loop':'3','Graph':'101000'})
    OL.setNumber('9Results/2M', attributes={'Graph':'010000'})
    OL.setNumber('9Results/4T', attributes={'Graph':'000100'})
else :
    OL.setNumber('1Geometry/X', attributes={'Loop':'0','Graph':'000000'})
    OL.setNumber('9Results/2M', attributes={'Graph':'000000'})
    OL.setNumber('9Results/4T', attributes={'Graph':'000000'})

if not OL.loop :
    OL.resetNumberChoices('1Geometry/X')
    OL.resetNumberChoices('9Results/2M')
    OL.resetNumberChoices('9Results/4T')
    
if OL.action == 'check' :
   exit(1)

if OL.isChanged('mesher') :
    OL.run('mesher', 'gmsh - ' +  onelab.path(__file__, 'beam.geo'),'-3')
    OL.mergeFile(onelab.path(__file__, 'beam.msh'))
    OL.call('ElmerGrid 14 2 ' + onelab.path(__file__,'beam') +' -out mesh', rundir=workdir)

if recomputeElmer :
    OL.call('ElmerSolver ' + onelab.path(__file__, 'beam.sif'), rundir=workdir)

VMIN = 1e3 * onelab.extract(onelab.path(__file__, 'beam.txt'), -1, 1)
VMAX = 1e3 * onelab.extract(onelab.path(__file__, 'beam.txt'), -1, 2)
OL.addNumberChoice('9Results/VMIN', VMIN)
OL.addNumberChoice('9Results/VMAX', VMAX)

cmd = 'gmsh ' + onelab.path(__file__,'beam.pos') + ' ' + onelab.path(__file__,'MNT.opt') + ' -'
OL.run('postpro',cmd)
  
M = onelab.extract(onelab.path(__file__, 'MNT.txt'), -1, 8)
N = onelab.extract(onelab.path(__file__, 'MNT.txt'), -1, 9)
T = onelab.extract(onelab.path(__file__, 'MNT.txt'), -1, 10)
OL.addNumberChoice('9Results/2M', M)
OL.addNumberChoice('9Results/3N', N)
OL.addNumberChoice('9Results/4T', T)

if not OL.batch :
    OL.sendCommand("Delete All;")
    OL.openProject(onelab.path(__file__, 'beam.pos'))
    OL.mergeFile(onelab.path(__file__, 'cut.opt'))
    OL.mergeFile(onelab.path(__file__, 'beam.opt'))
else :
    sys.stderr.write('%s: M = %12.5e (%12.5e, analytic 92)\n' %
                     (__file__, M, 9.56928e+01))
    sys.stderr.write('%s: T = %12.5e (%12.5e, analytic -368)\n' %
                     (__file__, T, -3.62325e+02))
    
# T(x) =-DENSITY * 9.81 * A * B * (L-x)/2
# M(x) = DENSITY * 9.81 * A * B * (L-x)^2/2
# T(L/2) = -368
# M(L/2) = 92
